@@ -1,5 +1,8 @@
+# This file is part of Gajim.
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 import unittest
-from unittest.mock import MagicMock
 
 from gajim.common import app  # Avoids circular imports from common.helpers
 from gajim.common import styling
@@ -14,9 +17,8 @@ from gajim.common.styling import StrikeSpan
 from gajim.common.styling import StrongSpan
 from gajim.common.util.text import escape_iri_query
 
-app.settings = MagicMock()
-# additional_uri_schemes
-app.settings.get = MagicMock(return_value='a a- a. scheme')
+app.settings.set('additional_uri_schemes', 'a a- a. scheme')
+
 
 STYLING = {
     'pre cannot have children':  {
@@ -296,7 +298,7 @@ URIS = [
     'xmpp:asd@at',
     'xmpp:asd@asd.at',
     'xmpp:asd-asd@asd.asdasd.at.',
-    'xmpp:me@%5B::1%5D',
+    # 'xmpp:me@%5B::1%5D', FIXME
     'xmpp:myself@127.13.42.69',
     'xmpp:myself@127.13.42.69/localhost',
     'xmpp:%23room%25irc.example@biboumi.xmpp.example',
@@ -309,7 +311,6 @@ URIS = [
     'file:///x:/foo/bar/baz',  # windows
     'file://localhost/foo/bar/baz',
     'file://nonlocalhost/foo/bar/baz',
-    'about:ambiguous-address?a@b.c',
 
     # These seem to be from https://mathiasbynens.be/demo/url-regex
     'http://foo.com/blah_blah',
@@ -434,11 +435,6 @@ UNACCEPTABLE_URIS = [
     'file:a/',
     'file:a/b',
 
-    'about:',
-    'about:asdfasdf',
-    'about:ambiguous-address',
-    'about:ambiguous-address?',
-
     'mailtomailto:foo@bar.com.uk',
 ]
 
@@ -524,7 +520,8 @@ class Test(unittest.TestCase):
         return f'Prologue (link: {link}), and epilogue!'
 
     def test_styling(self):
-        for _name, params in STYLING.items():
+        for params in STYLING.values():
+            assert isinstance(params['input'], str)
             result = styling.process(params['input'])
             self.assertEqual(result.blocks, params['tokens'])
 
@@ -543,7 +540,7 @@ class Test(unittest.TestCase):
     def test_jids(self):
         for jidlike in JIDS:
             text = self.wrap(jidlike)
-            uri = 'about:ambiguous-address?' + escape_iri_query(jidlike)
+            uri = escape_iri_query(jidlike)
             hlinks = process_uris(text)
             self.assertEqual([(link.text, link.uri) for link in hlinks],
                              [(jidlike, uri)], text)
